// File ERROR.C (used to build LIBBMP.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015

#include "libbmp.h"

typedef struct PTR
	{
	struct PTR	*	next ;
	IMAGE	*		image ;
	} POINTER ;

PRIVATE POINTER *pointers = NULL ;

void _AddPointer(IMAGE *image)
	{
	POINTER *pointer = (POINTER *) malloc(sizeof(POINTER)) ;
	if (pointer == NULL) _Error("Insufficient memory") ;
	pointer->image = image ;
	pointer->next = pointers ;
	pointers = pointer ;
	}

void _DelPointer(IMAGE *image)
	{
	POINTER *pointer, *previous ;

	previous = (POINTER *) &pointers ;
	for (pointer = pointers; pointer != NULL; pointer = pointer->next)
		{
		if (pointer->image == image)
			{
			previous->next = pointer->next ;
			free(pointer) ;
			return ;
			}
		previous = pointer ;
		}
	_Error("Invalid IMAGE pointer") ;
	}

void _ChkPointer(IMAGE *image)
	{
	POINTER *pointer ;

	for (pointer = pointers; pointer != NULL; pointer = pointer->next)
		{
		if (pointer->image == image) return ;
		}

	_Error("Invalid IMAGE pointer") ;
	}

void _Error(char *format, ...)
	{
	va_list args ;
	
	fprintf(stderr, "\n") ;
	va_start(args, format) ;
	vfprintf(stderr, format, args) ;
	va_end(args) ;
	fprintf(stderr, "\n") ;
	exit(255) ;
	}
