// File HSV2RGB.C (used to build LIBBMP.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015

#include "libbmp.h"

RGB HSV2RGB(HSV *hsv)
	{
	double r, g, b, f, p, q, t, h, s, v ;
	double *pRed[] = {&v, &q, &p, &p, &t, &v} ;
	double *pGrn[] = {&t, &v, &v, &q, &p, &p} ;
	double *pBlu[] = {&p, &p, &t, &v, &v, &q} ;
	unsigned val = hsv->val ;
	unsigned sat = hsv->sat ;
	unsigned hue = hsv->hue ;
	RGB rgb ;
	int i ;

	val = MIN(val, 100) ;
	if (sat == 0)
		{
		// achromatic (grey)
		rgb.red = rgb.grn = rgb.blu = (val * 255 + 50) / 100 ;
		return rgb ;
		}

	sat = MIN(sat, 100) ;
	s = sat / 100.0 ;
	h = (hue % 360) / 60.0 ;	// 0.0 <= h < 6.0
	i = (unsigned) h ;		//   0 <= i < 6
	f = h - (double) i ;		//   0 <= f < 1.0

	v = val / 100.0 ;
	p = v * (1.0 - s) ;
	q = v * (1.0 - s*f) ;
	t = v * (1.0 - s*(1.0 - f)) ;

	rgb.red = *pRed[i] * 255 + 0.5 ;
	rgb.grn = *pGrn[i] * 255 + 0.5 ;
	rgb.blu = *pBlu[i] * 255 + 0.5 ;

	return rgb ;
	}
