// Include file BMP.H
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015
//
// Purpose: Manipulation of 24-bit RGB BMP files using 
// direct access to the internal data structures.

#include <stdint.h>

typedef struct
	{
	uint16_t	hue ;	// 0 to 359 degrees
	uint8_t		sat ;	// 0 to 100 percent
	uint8_t		val ;	// 0 to 100 percent
	} HSV ;

typedef struct
	{
	uint8_t		blu ;	// 0 to 255
	uint8_t		grn ;	// 0 to 255
	uint8_t		red ;	// 0 to 255
	} RGB ;

typedef struct
	{
	unsigned 	rows ;
	unsigned 	cols ;
	unsigned 	dpi ;
	RGB			*pxlrow[0] ;
	} IMAGE ;

IMAGE *		NewImage(unsigned rows, unsigned cols) ;
IMAGE *		ReadBMP24(char *filespec) ;
void		WriteBMP24(char *filespec, IMAGE *image) ;
void		FreeImage(IMAGE *image) ;

unsigned	GetRows(IMAGE *image) ;
unsigned	GetCols(IMAGE *image) ;

unsigned	GetRed(IMAGE *image, unsigned row, unsigned col) ;
unsigned	GetGrn(IMAGE *image, unsigned row, unsigned col) ;
unsigned	GetBlu(IMAGE *image, unsigned row, unsigned col) ;

void		PutRed(IMAGE *image, unsigned row, unsigned col, unsigned red) ;
void		PutGrn(IMAGE *image, unsigned row, unsigned col, unsigned grn) ;
void		PutBlu(IMAGE *image, unsigned row, unsigned col, unsigned blu) ;

RGB			GetRGB(IMAGE *image, unsigned row, unsigned col) ;
void		PutRGB(IMAGE *image, unsigned row, unsigned col, RGB *rgb) ;

HSV 		RGB2HSV(RGB *rgb) ;
RGB 		HSV2RGB(HSV *hsv) ;
