// Main program file LAB5.C
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 2, 2015
//
// Purpose: Create double size and half size versions of an image.

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include "bmp.h"

IMAGE *		HalfSize(IMAGE *image) ;
IMAGE *		DoubleSize(IMAGE *image) ;

typedef struct
	{
	char	*format ;
	IMAGE	*(*function)() ;
	double	param ;
	} OPTION ;

OPTION *	GetOption(int argc, char **argv, OPTION options[]) ;
void		DisplayImage(char *filespec, char *format, ...) ;

int main(int argc, char **argv)
	{
	char *src_filespec, *dst_filespec ;
	static OPTION options[] =
		{
		{"Half Size",			HalfSize,			  0.0},
		{"Double Size",			DoubleSize,			  0.0},
		{NULL,					NULL,				  0.0}
		} ;
	OPTION *option ;
	IMAGE *image ;

	if (argc != 3 && argc != 4)
		{
		fprintf(stderr, "Usage: %s src-file dst-file {option#}\n", argv[0]) ;
		exit(255) ;
		}

	src_filespec = argv[1] ;
	dst_filespec = argv[2] ;

	image = ReadBMP24(src_filespec) ;

	if ((option = GetOption(argc, argv, options))== NULL)
		{
		fprintf(stderr, "Usage: %s src-file dst-file {option#}\n", argv[0]) ;
		exit(255) ;
		}

	image = (option->function)(image, option->param) ;

	WriteBMP24(dst_filespec, image) ;
	FreeImage(image) ;
	DisplayImage(dst_filespec, option->format, option->param) ;

	return 0 ;
	}

IMAGE *DoubleSize(IMAGE *src)
	{
	unsigned row, rows = 2 * src->rows ;
	unsigned col, cols = 2 * src->cols ;
	IMAGE *dst = NewImage(rows, cols) ;

	RGB **srcrow = src->pxlrow ;
	RGB **dstrow = dst->pxlrow ;
	for (row = 0; row < rows; row += 2)
		{
		RGB *srcpxl = *srcrow++ ;
		RGB *r0pxl = *dstrow++ ;
		RGB *r1pxl = *dstrow++ ;
		for (col = 0; col < cols; col += 2, srcpxl++)
			{
			r0pxl[0] = *srcpxl ;
			r0pxl[1] = *srcpxl ;
			r0pxl += 2 ;

			r1pxl[0] = *srcpxl ;
			r1pxl[1] = *srcpxl ;
			r1pxl += 2 ;
			}
		}

	FreeImage(src) ;
	return dst ;
	}

IMAGE *HalfSize(IMAGE *src)
	{
	unsigned row, rows = src->rows ;
	unsigned col, cols = src->cols ;
	IMAGE *dst = NewImage(rows/2, cols/2) ;

	RGB **srcrow = src->pxlrow ;
	RGB **dstrow = dst->pxlrow ;
	for (row = 1; row < rows; row += 2)
		{
		RGB *r0c0 = *srcrow++ ;
		RGB *r1c0 = *srcrow++ ;
		RGB *dstpxl = *dstrow++ ;
		for (col = 1; col < cols; col += 2, dstpxl++)
			{
			RGB *r0c1 = r0c0 + 1 ;
			RGB *r1c1 = r1c0 + 1 ;

			dstpxl->red = (r0c0->red + r0c1->red + r1c0->red + r1c1->red) / 4 ;
			dstpxl->grn = (r0c0->grn + r0c1->grn + r1c0->grn + r1c1->grn) / 4 ;
			dstpxl->blu = (r0c0->blu + r0c1->blu + r1c0->blu + r1c1->blu) / 4 ;

			r0c0 += 2 ;
			r1c0 += 2 ;
			}
		}

	FreeImage(src) ;
	return dst ;
	}

OPTION *GetOption(int argc, char **argv, OPTION options[])
	{
	unsigned number, index = 0 ;

	// count number of options
	for (number = 0; options[number].function != NULL; number++) ;

	if (argc == 4)
		{
		sscanf(argv[3], "%u", &index) ;
		}
	else
		{
		printf("\nOptions:\n\n") ;
		for (index = 0; index < number; index++)
			{
			char title[100] ;
			sprintf(title, options[index].format, options[index].param) ;
			printf("%2d: %s\n", index + 1, title) ;
			}

		printf("\nOption? ") ;
		scanf("%u", &index) ;
		}

	return (1 <= index && index <= number) ? &options[index - 1] : NULL ;
	}

void DisplayImage(char *filespec, char *format, ...)
	{
	char progspec[200], command[1000], *program_files, title[100] ;
	struct stat filestat ;
	va_list args ;
	
	va_start(args, format) ;
	vsprintf(title, format, args) ;
	va_end(args) ;

#if defined(_WIN32)
	program_files = getenv("ProgramFiles(x86)") ;
	if (program_files == NULL) program_files = getenv("C:\\Program Files") ;
	if (program_files == NULL) program_files = "" ;

	sprintf(progspec, "%s\\IrfanView\\i_view32.exe", program_files) ;
	if (stat(progspec, &filestat) == 0 && (filestat.st_mode & S_IFREG) != 0)
		{
		sprintf(command, "\"%s\" %s /hide=7 /title=%s", progspec, filespec, title) ;
		}
	else sprintf(command, "mspaint \"%s\"\n", filespec) ;
#elif defined(__unix)
	sprintf(command, "qiv \"%s\"", filespec) ;
#elif defined(__APPLE__)
	sprintf(command, "open \"%s\"", filespec) ;
#endif 
	system(command) ;
	}

