// File WRITEFILE.C (used to build LIBBMP.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015

#include "libbmp.h"

PRIVATE int		_WriteHeader(FILE *fp, IMAGE *image) ;
PRIVATE unsigned		_TotalPixelBytes(IMAGE *image) ;

void WriteBMP24(char *filespec, IMAGE *image)
	{
	unsigned row_bytes ;
	FILE *fp ;
	int row ;

	_ChkPointer(image) ;
	if ((fp = fopen(filespec, "wb")) == NULL)
		_Error("Error creating file %s", filespec) ;

	if (_WriteHeader(fp, image) != 1)
		_Error("Error writing header of file %s", filespec) ;

	row_bytes = _BytesPerRow(image->cols) ;
	for (row = 0; row < image->rows; row++)
		{
		if (fwrite(image->pxlrow[row], row_bytes, 1, fp) != 1)
			_Error("Error writing pixels to file %s", filespec) ;
		}

	fclose(fp) ;
	}

PRIVATE int _WriteHeader(FILE *fp, IMAGE *image)
	{
	BMP_HEADER header ;

	header.type = 0x4D42 ;							// "magic" BMP signature
	header.size = sizeof(BMP_HEADER) + _TotalPixelBytes(image) ;
	header.reserved1 = 0 ;
	header.reserved2 = 0 ;
	header.offset = sizeof(BMP_HEADER) ;
	header.header_size = 40 ;
	header.width = image->cols ;
	header.height = image->rows ;
	header.planes = 1 ;
	header.bits = 3 * 8 ;							// 1 byte per primary color
	header.compression = 0 ;						// no compression
	header.imagesize = _TotalPixelBytes(image) ;
	header.xresolution = 0.5 + INCHES_PER_METER * image->dpi ;
	header.yresolution = 0.5 + INCHES_PER_METER * image->dpi ;
	header.ncolours = 0 ;							// ignored 
	header.importantcolours = 0 ;					// ignored

	return fwrite(&header, sizeof(BMP_HEADER), 1, fp) ;
	}

PRIVATE unsigned _TotalPixelBytes(IMAGE *image)
	{
	return image->rows * _BytesPerRow(image->cols) ;
	}

